#ifndef __CON_MAIN_H__
#define __CON_MAIN_H__
//****************************************************************************
//**
//**    CON_MAIN.H
//**    Header - Command Console
//**
//**    Copyright (c) 1998 Chris Hargrove
//**    Notice: Usage of any code in this file is subject to the rules
//**            described in the LICENSE.TXT file included in this directory.
//**            Reading, compiling, or otherwise using this code constitutes
//**            automatic acceptance of the rules in said text file.
//**
//****************************************************************************
//============================================================================
//    INTERFACE REQUIRED HEADERS
//============================================================================
#include "sys_defs.h"
//============================================================================
//    INTERFACE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
//============================================================================
#define CON_FUNC(name, flags) \
	void name##_confunc_body(con_var_t *argvar, int argc, char **argv); \
	con_func_t name##_confunc(#name, name##_confunc_body, flags, false); \
	void name##_confunc_body(con_var_t *argvar, int argc, char **argv)

#define CON_VARHANDLER(type) \
	void handler_##type##_confunc_body(con_var_t *argvar, int argc, char **argv); \
	con_func_t handler_##type##_confunc(#type, handler_##type##_confunc_body, 0, true); \
	void handler_##type##_confunc_body(con_var_t *argvar, int argc, char **argv)

#define CON_VAR(type, name, ivalue, flags) \
	type name = ivalue; \
	con_var_t name##_convar(#name, &##name, #type, flags)

// console function flags
enum
{
    CONFUNCF_RESTRICTED     = 0x00000001    // func is not accessible during restricted execute
};

// console variable flags
enum
{
    CONVARF_RESTRICTED      = 0x00000001   // var is not accessible during restricted execute
};

//============================================================================
//    INTERFACE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
//============================================================================
class con_func_t;
class con_var_t;

typedef void (*con_func_bodyFunc_t)(con_var_t*, int, char**);

//============================================================================
//    INTERFACE STRUCTURES / UTILITY CLASSES
//============================================================================
//============================================================================
//    INTERFACE DATA DECLARATIONS
//============================================================================
//============================================================================
//    INTERFACE FUNCTION PROTOTYPES
//============================================================================

// initialization/shutdown
void CON_Init();
void CON_Shutdown();

// var/func operations
void CON_RegisterVar(con_var_t* var); // adds a variable to the console variable list
void CON_RegisterFunc(con_func_t* func); // adds a function to the console function list
void CON_RegisterVarHandler(con_func_t* varHandler); // adds a variable handler to the handler list
con_func_t* CON_GetHandlerForVarType(char* varTypeName); // returns handler for a named variable type

// command execution
bbool CON_Execute(bbool restricted, char* fmt, ... ); // executes an arbitrary command string
bbool CON_ExecuteFile(char* fileName); // executes a series of commands from a file
char* CON_MatchCommand(char* nameStart, dword nameCmpLen, dword matchSkipCount); // finds a command beginning with the given string

// printing
void CON_Printf(char* fmt, ... ); // prints an arbitrary string to the print buffer
dword CON_GetNumPrintLines(); // returns number of lines in the print buffer
char* CON_GetPrintLine(dword index); // returns the string for a given print line, n lines backward from the most recent line
void CON_SetPrintCallback(void (*callback)(char*)); // set a function that will be called along with CON_Printf

//============================================================================
//    INTERFACE OBJECT CLASS DEFINITIONS
//============================================================================

// console function
class con_func_t
{
public:
    const char* name; // name of console function
    con_func_bodyFunc_t func; // function body to call
    dword flags; // CONFUNCF_ flags
    con_func_t* next; // next console function in list

    con_func_t(const char* inName, con_func_bodyFunc_t inFunc, dword inFlags, bbool isVarHandler);
    ~con_func_t();
};

// console variable
class con_var_t
{
public:
    const char* name; // name of console variable
    const char* varTypeName; // name of variable type
    void* valuePtr; // pointer to actual data
    dword flags; // CONVARF_ flags
    con_func_t* handler; // console function that acts as handler
    con_var_t* next; // next console variable in list

    con_var_t(const char* inName, void* inValuePtr, const char* inVarTypeName, dword inFlags);
    ~con_var_t();
};

//============================================================================
//    INTERFACE TRAILING HEADERS
//============================================================================

//****************************************************************************
//**
//**    END HEADER CON_MAIN.H
//**
//****************************************************************************
#endif // __CON_MAIN_H__
